# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Alert',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, primary_key=True, serialize=False)),
                ('timestamp', models.DateTimeField(auto_now_add=True)),
                ('state', models.PositiveSmallIntegerField(default=0, verbose_name='State')),
            ],
            options={
                'verbose_name_plural': 'Alerts',
                'ordering': ('-timestamp',),
                'verbose_name': 'Alert',
            },
        ),
        migrations.CreateModel(
            name='Observer',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('operator', models.CharField(choices=[('EqualOperator', 'Equals to value'), ('GreaterThanOperator', 'Greater than value'), ('LowerThanOperator', 'Lower than value')], verbose_name='Operator type', max_length=32)),
                ('value', models.PositiveIntegerField(verbose_name='Value')),
                ('waiting_period', models.PositiveIntegerField(help_text='In seconds', verbose_name='Waiting period')),
            ],
            options={
                'verbose_name_plural': 'Observers',
                'ordering': ('name',),
                'verbose_name': 'Observer',
            },
        ),
        migrations.AddField(
            model_name='alert',
            name='observer',
            field=models.ForeignKey(to='howl.Observer', verbose_name='Observer'),
        ),
    ]



from django.utils.translation import ugettext_lazy as _


def add_operator_type(operator_class):
    # Fail silently if we try to double register operator classes.
    if operator_class.__name__ in add_operator_type._REGISTRY:
        return False

    add_operator_type._REGISTRY[operator_class.__name__] = operator_class
    return True

add_operator_type._REGISTRY = {}


def get_operator_types():
    return sorted([
        (operator_class.__name__, operator_class.display_name)
        for operator_class in add_operator_type._REGISTRY.values()
    ])


def get_operator_class(class_name):
    cls = add_operator_type._REGISTRY.get(class_name, None)

    if not cls:
        raise KeyError('Key "{0}" not in operator registry'.format(class_name))

    return cls


class OperatorException(Exception):
    pass


class BaseOperator(object):
    display_name = None

    @classmethod
    def register(cls):
        return add_operator_type(cls)

    def __init__(self, observer):
        self.observer = observer

    def compare(self, compare_value):
        raise NotImplementedError('Method compare is not implemented.')


class EqualOperator(BaseOperator):
    display_name = _('Equals to value')

    def compare(self, compare_value):
        return self.observer.value == compare_value

EqualOperator.register()


class LowerThanOperator(BaseOperator):
    display_name = _('Lower than value')

    def compare(self, compare_value):
        return compare_value < self.observer.value

LowerThanOperator.register()


class GreaterThanOperator(BaseOperator):
    display_name = _('Greater than value')

    def compare(self, compare_value):
        return compare_value > self.observer.value

GreaterThanOperator.register()



from datetime import timedelta

from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from .operators import get_operator_class, get_operator_types
from .signals import alert_clear, alert_notify, alert_wait


class Observer(models.Model):
    name = models.CharField(_('Name'), max_length=255)
    operator = models.CharField(
        _('Operator type'), max_length=32, choices=get_operator_types())
    value = models.PositiveIntegerField(_('Value'))
    waiting_period = models.PositiveIntegerField(_('Waiting period'), help_text=_('In seconds'))
    alert_every_time = models.BooleanField(_('Alert every time'), default=False)

    class Meta:
        verbose_name = _('Observer')
        verbose_name_plural = _('Observers')
        ordering = ('name',)

    def __str__(self):
        return self.name

    def compare(self, compare_value):
        operator_class = get_operator_class(self.operator)

        if operator_class(self).compare(compare_value):
            Alert.clear(self)
            return True

        Alert.set(self, compare_value)
        return False


class Alert(models.Model):
    STATE_WAITING, STATE_NOTIFIED = range(0, 2)
    STATE_CHOICES = (
        (STATE_WAITING, _('Waiting')),
        (STATE_NOTIFIED, _('Notified')),
    )

    observer = models.ForeignKey(Observer, verbose_name=_('Observer'))
    timestamp = models.DateTimeField(auto_now_add=True)
    value = models.CharField(_('Value'), max_length=255)
    state = models.PositiveSmallIntegerField(
        _('State'), choices=STATE_CHOICES, default=STATE_WAITING)

    class Meta:
        verbose_name = _('Alert')
        verbose_name_plural = _('Alerts')
        ordering = ('-timestamp',)

    def __str__(self):
        return _('Alert (ID: {0}) for {1}').format(self.pk, self.observer.name)

    @classmethod
    def set(cls, observer, compare_value):
        obj, created = Alert.objects.get_or_create(
            observer=observer, defaults={'value': compare_value})

        if created:
            alert_wait.send(sender=cls, instance=obj)
            return True

        alert_time = obj.timestamp + timedelta(seconds=observer.waiting_period)

        if alert_time < timezone.now():
            if obj.state == obj.STATE_NOTIFIED and observer.alert_every_time:
                obj.value = compare_value
                obj.save(update_fields=['value'])
                alert_notify.send(sender=cls, instance=obj)

            if obj.state == obj.STATE_WAITING:
                obj.value = compare_value
                obj.state = obj.STATE_NOTIFIED
                obj.save(update_fields=['value', 'state'])
                alert_notify.send(sender=cls, instance=obj)

    @classmethod
    def clear(cls, observer):
        try:
            obj = Alert.objects.get(observer=observer)
            alert_clear.send(sender=cls, instance=obj)
            obj.delete()
        except Alert.DoesNotExist:
            pass



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from .models import Alert, Observer


@admin.register(Observer)
class ObserverAdmin(admin.ModelAdmin):
    list_display = ('name', 'operator', 'value')


@admin.register(Alert)
class AlertAdmin(admin.ModelAdmin):
    list_display = ('get_observer_name', 'timestamp', 'value', 'state')
    list_filter = ('state',)

    def get_observer_name(self, obj):
        return obj.observer.name

    get_observer_name.short_description = _('Observer')
